/***************************************************************************
 *   Copyright (C) 2006 by Gerd v. Egidy                                   *
 *   gve@intra2net.com                                                     *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License version   *
 *   2.1 as published by the Free Software Foundation.                     *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU Lesser General Public License for more details.                   *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef __LIBT2N_CLIENT
#define __LIBT2N_CLIENT

#include <string>
#include <vector>
#include <list>

#include <boost/function.hpp>

#include "connection.hxx"
#include "types.hxx"

namespace libt2n
{

/** @brief a (generic) connection from client to server. Abstract.
 */
class client_connection : public connection
{
    private:
        log_level_values log_level;
        std::ostream *logstream;

        /// vector initialized for all callback-types, all elements in each list will be called
        std::vector<std::list<boost::function<void ()> > > callbacks;

    protected:
        void do_callbacks(callback_event_type event);

    public:
        client_connection();

        virtual ~client_connection();

        void close();

        void set_logging(std::ostream *_logstream, log_level_values _log_level);

        std::ostream* get_logstream(log_level_values level);

        void add_callback(callback_event_type event, const boost::function<void ()>& func);
};

}

#endif
