/***************************************************************************
 *   Copyright (C) 2006 by Gerd v. Egidy                                   *
 *   gve@intra2net.com                                                     *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License version   *
 *   2.1 as published by the Free Software Foundation.                     *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU Lesser General Public License for more details.                   *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <string>
#include <sstream>
#include <iostream>

#include <netinet/in.h>

#include "connection.hxx"

namespace libt2n
{

connection::packet_size_indicator connection::bytes_available()
{
    // no size information -> no packet
    if (buffer.size() < sizeof(packet_size_indicator))
        return 0;

    packet_size_indicator psize=ntohl(*((packet_size_indicator*)(buffer.data())));

    // enough data for one packet in buffer?
    if (buffer.size() < sizeof(packet_size_indicator)+psize)
        return 0;

    // ok, full packet there
    return psize;
}

/** @brief read a complete data packet from the buffer. The packet is removed from the
            connection buffer.
    @param[out] data the data package
    @retval true if packet found
*/
bool connection::get_packet(std::string& data)
{
    packet_size_indicator psize;

    if ((psize=bytes_available()))
    {
        data.assign(buffer,sizeof(packet_size_indicator),psize);
        buffer.erase(0,sizeof(packet_size_indicator)+psize);
        return true;
    }
    else
        return false;
}

/** @brief get (maybe incomplete) data of the next packet from the buffer. Does not remove the data
            from the connection buffer.
    @param[out] data the data package
    @retval full size of the packet when it will be complete
*/
unsigned int connection::peek_packet(std::string& data)
{
    // no size information -> no packet
    if (buffer.size() < sizeof(packet_size_indicator))
        return 0;

    packet_size_indicator psize=ntohl(*((packet_size_indicator*)(buffer.data())));

    // not the full data available?
    packet_size_indicator currsize=psize;
    if (buffer.size() < currsize+sizeof(packet_size_indicator))
        currsize=buffer.size()-sizeof(packet_size_indicator);

    data.assign(buffer,sizeof(packet_size_indicator),currsize);

    return psize;
}

/// send a blob to the peer
void connection::write(const std::string& data)
{
    // prepend packet size to data
    packet_size_indicator psize=htonl(data.size());
    std::string send_data(data);
    send_data.insert(0,(char*)&psize,sizeof(packet_size_indicator));

    real_write(send_data);
}

}
